package org.galaxy.modelfragment.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.galaxy.modelfragment.schema.ModelFragment;


/**
 * Utils for ModelFragment manipulation
 * @author ebrosse
 *
 */
public class ModelFragmentUtils {

    private static void unzipEntry(ZipFile zipfile, ZipEntry entry, File outputDir) throws IOException {


        if (entry.isDirectory()) {
            createDirectory(new File(outputDir, entry.getName()));
            return;
        }

        File outputFile = new File(outputDir, entry.getName());
        if (!outputFile.getParentFile().exists()){
            createDirectory(outputFile.getParentFile());
        }

        BufferedInputStream inputStream = new BufferedInputStream(zipfile.getInputStream(entry));
        BufferedOutputStream outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));

        try {
            copy(inputStream, outputStream);
        } finally {
            outputStream.close();
            inputStream.close();
        }
    }

    private static void createDirectory(File directory) {
        if(!directory.mkdirs()) throw new RuntimeException("Can not create the directory "+directory);
    }


    private static void copy(InputStream in, OutputStream out) throws IOException {
        if (in == null)
            throw new NullPointerException("InputStream is null!");
        if (out == null)
            throw new NullPointerException("OutputStream is null");

        // Transfer bytes from in to out
        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0)
        {
            out.write(buf, 0, len);
        }
        in.close();
        out.close();
    }
    
    /**
     * Unzips an given archive into the target outpurDir folder and returns the ModelFragment
     * @param archive  : the given archive
     * @param outputDir : the target output folder      
     * @return the loaded ModelFragment
     * @throws ZipException : ZipException
     * @throws IOException : IOException
     */
    public static ModelFragment unpackageFragment(File archive, File outputDir) throws ZipException, IOException{
        
        ZipFile zipfile = new ZipFile(archive);

        for (Enumeration<? extends ZipEntry> e = zipfile.entries(); e.hasMoreElements(); ) {
            ZipEntry entry = e.nextElement();
            unzipEntry(zipfile, entry, outputDir);
        }
        
        File metadatas = new File(outputDir.getAbsolutePath() + File.separator + "metadatas.xml");
        if (metadatas.exists())
            return loadMetadata(metadatas);
        
        return null;
    }
    
    /**
     * Serializes a given fragment data in the metadatas file
     * @param fragment : the ModelFragment to serialize 
     * @param metadatasFile : the meta
     * @throws JAXBException JAXBException
     */
    public static void createMetadata(ModelFragment fragment, File metadatasFile) throws JAXBException{

        JAXBContext jaxbContext = JAXBContext.newInstance(ModelFragment.class);
        Marshaller marshaller = jaxbContext.createMarshaller();
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

        marshaller.marshal(fragment, metadatasFile);
    }
    

    /**
     * Loads the ModelFragment from a given metadatas.xml file
     * @param metadatas : the metadatas.xml file
     * @return the loaded ModelFragment
     */
    public static ModelFragment loadMetadata(File metadatas) {
        if (metadatas.exists()){
            try {
                JAXBContext jc = JAXBContext.newInstance(ModelFragment.class);

                Unmarshaller unmarshaller = jc.createUnmarshaller();
                return (ModelFragment) unmarshaller.unmarshal(metadatas);

            } catch (JAXBException e) {
                e.printStackTrace(System.err);
            }
        }
        return null;

    }


}

