package org.galaxy.service.projectmanagement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.validation.constraints.NotNull;

import org.galaxy.i18n.Messages;
import org.galaxy.model.EParticipant;
import org.galaxy.model.EProject;
import org.galaxy.model.EProjectParticipant;
import org.galaxy.model.ERole;

/**
 * Implementation of IProjectManagement services dedicated to projects management 
 */
@Remote
@Stateless
@WebService(targetNamespace = "http://org.galaxy.service.projectmanagement/v1/", name = "ProjectManagement", portName = "ProjectManagementPort", serviceName = "ProjectManagementService")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, use = SOAPBinding.Use.LITERAL, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class ProjectManagement implements IProjectManagement {

	@PersistenceContext(unitName = "galaxy")
	private EntityManager manager;

	@Override
	@WebMethod
	public void addParticipant(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "participantId") Integer participantId,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Participant"));
		} else if (roleIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		EParticipant participant = manager.find(EParticipant.class,
				participantId);

		if (participant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Participant", participantId.toString()));
		}

		for (Integer roleId : roleIds) {

			ERole role = manager.find(ERole.class, roleId);

			if (role == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Role", roleId.toString()));
			}

			EProjectParticipant relation = new EProjectParticipant();
			relation.setParticipant(participant);
			relation.setRole(role);
			relation.setProject(project);
			manager.persist(relation);

		}
	}

	@Override
	@WebMethod
	public void removeParticipant(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "participantId") Integer participantId,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Participant"));
		} else if (roleIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		EParticipant participant = manager.find(EParticipant.class,
				participantId);

		if (participant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Participant", participantId.toString()));
		}
		
		for (Integer roleId : roleIds) {
			ERole role = manager.find(ERole.class, roleId);

			if (role == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Role", roleId.toString()));
			}
			
			EProjectParticipant relation = new EProjectParticipant();
			relation.setParticipant(participant);
			relation.setRole(role);
			relation.setProject(project);
			
			relation = manager.merge(relation);
			manager.remove(relation);
		}
	}

	@Override
	@WebMethod
	public void addRole(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "roleId") Integer roleId)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (roleId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		ERole role = manager.find(ERole.class, roleId);

		if (role == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING", "Role",
					roleId.toString()));
		}

		project.getRoles().add(role);
		role.getProjects().add(project);
		manager.merge(project);
		manager.merge(role);
	}

	@Override
	@WebMethod
	public void removeRole(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "roleId") Integer roleId)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (roleId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		ERole role = manager.find(ERole.class, roleId);

		if (role == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING", "Role",
					roleId.toString()));
		}

		project.getRoles().remove(role);
		role.getProjects().remove(project);

		manager.merge(project);
		manager.merge(role);

	}

	@Override
	@WebMethod
	@WebResult(name = "project")
	public Project findProjectById(
			@NotNull @WebParam(name = "projectId") Integer projectId)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}

		EProject eproject = manager.find(EProject.class, projectId);

		if (eproject == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		Project project = new Project();
		project.setName(eproject.getName());
		project.setId(eproject.getProjectId());
		project.setDescription(eproject.getDescription());			
		project.setStartDate(eproject.getStartDate());
		project.setEndDate(eproject.getEndDate());
	

		return project;
	}

	@Override
	@WebMethod
	@WebResult(name = "projects")
	public List<Project> findProjectByIds(
			@NotNull @WebParam(name = "projectIds") List<Integer> projectIds)
			throws IOException {

		List<Project> result = new ArrayList<Project>();
		if (projectIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}

		for (Integer projectId : projectIds) {
			EProject eproject = manager.find(EProject.class, projectId);

			if (eproject == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Project", projectId.toString()));
			}

			Project project = new Project();
			project.setName(eproject.getName());
			project.setId(eproject.getProjectId());
			project.setDescription(eproject.getDescription());			
			project.setStartDate(eproject.getStartDate());
			project.setEndDate(eproject.getEndDate());
			result.add(project);

		}

		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "projects")
	public List<Project> findAllProjects() throws IOException {

		List<Project> result = new ArrayList<Project>();
		Query query = manager.createQuery("SELECT p FROM EProject p");

		for (EProject eproject : (Collection<EProject>) query.getResultList()) {
			Project project = new Project();
			project.setName(eproject.getName());
			project.setId(eproject.getProjectId());
			project.setDescription(eproject.getDescription());			
			project.setStartDate(eproject.getStartDate());
			project.setEndDate(eproject.getEndDate());
			result.add(project);
		}

		return result;
	}

	@Override
	@WebMethod
	public void addParticipants(
			Integer projectId,
			@NotNull @WebParam(name = "participantIds") List<Integer> participantIds,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (participantIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Participant"));
		} else if (roleIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		for (Integer participantId : participantIds) {
			EParticipant participant = manager.find(EParticipant.class,
					participantId);

			if (participant == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Participant", participantId.toString()));
			}

			for (Integer roleId : roleIds) {

				ERole role = manager.find(ERole.class, roleId);

				if (role == null) {
					throw new IOException(Messages.getString("GALAXY_MISSING",
							"Role", roleId.toString()));
				}

				EProjectParticipant relation = new EProjectParticipant();
				relation.setParticipant(participant);
				relation.setRole(role);
				relation.setProject(project);
				manager.persist(relation);

			}
		}
	}

	@Override
	@WebMethod
	public void removeParticipants(
			Integer projectId,
			@NotNull @WebParam(name = "participantIds") List<Integer> participantIds,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds) throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (participantIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID",
					"Participant"));
		} else if (roleIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		for (Integer participantId : participantIds) {
			EParticipant participant = manager.find(EParticipant.class,
					participantId);

			if (participant == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Participant", participantId.toString()));
			}

			for (Integer roleId : roleIds) {
				ERole role = manager.find(ERole.class, roleId);

				if (role == null) {
					throw new IOException(Messages.getString("GALAXY_MISSING",
							"Role", roleId.toString()));
				}

				EProjectParticipant relation = new EProjectParticipant();
				relation.setParticipant(participant);
				relation.setRole(role);
				relation.setProject(project);
				
				relation = manager.merge(relation);
				manager.remove(relation);
			}
		}
	}

	@Override
	@WebMethod
	public void addRoles(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds) throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (roleIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		for(Integer roleId : roleIds){
			ERole role = manager.find(ERole.class, roleId);

			if (role == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING", "Role",
						roleId.toString()));
			}
			project.getRoles().add(role);
			role.getProjects().add(project);
			manager.merge(role);
		}
		
		manager.merge(project);
	}

	@Override
	@WebMethod
	public void removeRoles(
			@NotNull @WebParam(name = "projectId") Integer projectId,
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds) throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		} else if (roleIds == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		
		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}
		
		for(Integer roleId : roleIds){
			ERole role = manager.find(ERole.class, roleId);

			if (role == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING", "Role",
						roleId.toString()));
			}
			project.getRoles().remove(role);
		}
		manager.merge(project);

	}

	@Override
	@WebMethod
	@WebResult(name = "projects")
	public List<Project> findProjectsByParticipant(
			@NotNull @WebParam(name = "participantId") Integer participantId)
			throws IOException {
		
		List<Project> result = new ArrayList<Project>();
		if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Participant"));
		}

		EParticipant participant = manager.find(EParticipant.class, participantId);

		if (participant == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", participant.toString()));
		}
		
		Query query = manager.createQuery("SELECT p FROM EProject p,EProjectParticipant r where r.project = p and r.participant = :participant").setParameter("participant", participant);

		for (EProject eproject : (Collection<EProject>) query.getResultList()) {
			Project project = new Project();
			project.setName(eproject.getName());
			project.setId(eproject.getProjectId());
			project.setDescription(eproject.getDescription());			
			project.setStartDate(eproject.getStartDate());
			project.setEndDate(eproject.getEndDate());
			result.add(project);
		}
		
//		for(EProjectParticipant relation : participant.getParticipation()){
//			EProject eproject = relation.getProject();	
//			Project project = new Project();
//			project.setName(eproject.getName());
//			project.setId(eproject.getProjectId());
//			project.setDescription(eproject.getDescription());			
//			project.setStartDate(eproject.getStartDate());
//			project.setEndDate(eproject.getEndDate());
//
//			result.add(project);	
//		}

		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "projectId")
	public Integer createProject(
			@NotNull @WebParam(name = "project") Project project)
			throws IOException {

		// Parameter Validation
		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_PV_NULL",
					"Project"));
		}

		EProject eproject = new EProject();
		eproject.setName(project.getName());
		eproject.setDescription(project.getDescription());
		eproject.setStartDate(project.getStartDate());
		eproject.setEndDate(project.getEndDate());
		
		manager.persist(eproject);

		return eproject.getProjectId();
	}

	@Override
	@WebMethod
	@WebResult(name = "projectIds")
	public void deleteProjects(
			@NotNull @WebParam(name = "projectIds") List<Integer> projectIds)
			throws IOException {
		for (Integer projectId : projectIds) {
			if (projectId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID",
						"Project"));
			}

			EProject project = manager.find(EProject.class, projectId);

			if (project == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Project", projectId.toString()));
			}

			manager.remove(project);
		}
	}

	@Override
	@WebMethod
	public void updateProject(
			@NotNull @WebParam(name = "project") Project project)
			throws IOException {
		// Parameter Validation
		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_PV_NULL",
					"Project"));
		} else if (project.getId() == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID"));
		}

		EProject eproject = new EProject();
		eproject.setName(project.getName());
		eproject.setDescription(project.getDescription());
		eproject.setProjectId(project.getId());
		eproject.setStartDate(project.getStartDate());
		eproject.setEndDate(project.getEndDate());

		if (findProjectById(eproject.getProjectId()) == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"PROJECT", project.getId().toString()));
		}

		manager.merge(eproject);

	}

	@Override
	@WebMethod
	public void deleteProject(
			@NotNull @WebParam(name = "projectId") Integer projectId)
			throws IOException {

		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		manager.remove(project);

	}

}
