package org.galaxy.service.rolemanagement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.validation.constraints.NotNull;

import org.galaxy.i18n.Messages;
import org.galaxy.model.EParticipant;
import org.galaxy.model.EProject;
import org.galaxy.model.EProjectParticipant;
import org.galaxy.model.ERole;


/**
 * Implementation of IUserManagement services dedicated to role management 
 */
@Remote
@Stateless
@WebService(targetNamespace = "http://org.galaxy.service.rolemanagement/v1/", name = "RoleManagement", portName = "RoleManagementPort", serviceName = "RoleManagementService")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, use = SOAPBinding.Use.LITERAL, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class RoleManagement implements IRoleManagement{

	
	@PersistenceContext(unitName = "galaxy")
	private EntityManager manager;
	
	@Override
    @WebMethod
    @WebResult(name = "roles")
	public List<Role> findRolesByProject(
			@NotNull @WebParam(name = "projectId") Integer projectId) throws IOException{

		List<Role> result = new ArrayList<Role>();
		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}

		EProject project = manager.find(EProject.class, projectId);

		if (project == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Project", projectId.toString()));
		}

		for(ERole erole : project.getRoles()){
			Role role = new Role();
			role.setName(erole.getName());
			role.setId(erole.getRoleId());
			role.setDescription(erole.getDescription());
			role.setCategory(erole.getCategory());;
			
			result.add(role);
		}


		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "roles")
	public List<Role> findAllRoles() throws IOException{
		List<Role> result = new ArrayList<Role>();
		Query query = manager.createQuery("SELECT r FROM ERole r");

		for (ERole erole : (Collection<ERole>) query.getResultList()) {
			Role role = new Role();
			role.setName(erole.getName());
			role.setId(erole.getRoleId());
			role.setDescription(erole.getDescription());
			role.setCategory(erole.getCategory());
			result.add(role);
		}

		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "role")
	public Role findRoleById( 
			@NotNull @WebParam(name = "roleId") Integer roleId) throws IOException{

		if (roleId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
		}

		ERole erole = manager.find(ERole.class, roleId);

		if (erole == null) {
			throw new IOException(Messages.getString("GALAXY_MISSING",
					"Role", roleId.toString()));
		}

		Role role = new Role();
		role.setName(erole.getName());
		role.setId(erole.getRoleId());
		role.setDescription(erole.getDescription());
		role.setCategory(erole.getCategory());

		return role;
	}

	@Override
	@WebMethod
	@WebResult(name = "roles")
	public List<Role> findRolesByProjectAndParticipant(
			@NotNull @WebParam(name = "projectId") Integer projectId, 
			@NotNull @WebParam(name = "participantId") Integer participantId) throws IOException{
		
		if (projectId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Project"));
		}
				
		if (participantId == null) {
			throw new IOException(Messages.getString("GALAXY_PV_ID", "Participant"));
		}
		
		
		EParticipant participant = manager.find(EParticipant.class, participantId);
		EProject project = manager.find(EProject.class, projectId);

		
		List<Role> result = new ArrayList<Role>();
		Query query = manager.createQuery("SELECT p FROM EProjectParticipant p where p.participant = :participant AND p.project = :project").setParameter("participant", participant).setParameter("project",project);

		for (EProjectParticipant relation : (Collection<EProjectParticipant>) query.getResultList()) {
			ERole erole = relation.getRole();
			
			Role role = new Role();
			role.setName(erole.getName());
			role.setId(erole.getRoleId());
			role.setDescription(erole.getDescription());
			role.setCategory(erole.getCategory());
			result.add(role);
		}

		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "roles")
	public List<Role> findRoleByIds(
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds) throws IOException {
		
		
		List<Role> result = new ArrayList<Role>();
		for(Integer roleId : roleIds){
			if (roleId == null) {
				throw new IOException(Messages.getString("GALAXY_PV_ID", "Role"));
			}

			ERole erole = manager.find(ERole.class, roleId);

			if (erole == null) {
				throw new IOException(Messages.getString("GALAXY_MISSING",
						"Role", roleId.toString()));
			}

			Role role = new Role();
			role.setName(erole.getName());
			role.setId(erole.getRoleId());
			role.setDescription(erole.getDescription());
			role.setCategory(erole.getCategory());
			
			result.add(role);
		}


		return result;
	}

	@Override
	@WebMethod
	@WebResult(name = "roleId")
	public Integer createRole(
			@NotNull @WebParam(name = "role")Role role) throws IOException{
		
		// Parameter Validation
		if(role == null){
			throw  new IOException(Messages.getString("GALAXY_PV_NULL","Role"));
		}
	
		ERole erole = new ERole();
		erole.setName(role.getName());
		erole.setDescription(role.getDescription());
		erole.setCategory(role.getCategory());
		manager.persist(erole);
		
		return erole.getRoleId();
	}

	@Override
	@WebMethod
	public void deleteRole(
			@NotNull @WebParam(name = "roleId")Integer roleId) throws IOException{
	
		if(roleId == null){
			throw  new IOException(Messages.getString("GALAXY_PV_ID","Role"));
		}
		
		ERole role = manager.find(ERole.class, roleId);
		
		if(role == null){
			throw  new IOException(Messages.getString("GALAXY_MISSING","Role",roleId.toString()));
		}
		
		manager.remove(role);
		
	}

	@Override
	@WebMethod
	@WebResult(name = "roleIds")
	public void deleteRoles(
			@NotNull @WebParam(name = "roleIds") List<Integer> roleIds) throws IOException{
	
		for(Integer roleId : roleIds){
			if(roleId == null){
				throw  new IOException(Messages.getString("GALAXY_PV_ID","Role"));
			}
			
			ERole role = manager.find(ERole.class, roleId);
			
			if(role == null){
				throw  new IOException(Messages.getString("GALAXY_MISSING","Role",roleId.toString()));
			}
			
			manager.remove(role);
		}	
	}

	@Override
	@WebMethod
	public void updateRole(
			@NotNull @WebParam(name = "role")Role role) throws IOException{
		
		// Parameter Validation
		if(role == null){
			throw  new IOException(Messages.getString("GALAXY_PV_NULL","Role"));
		}else if(role.getId() == null){
			throw  new IOException(Messages.getString("GALAXY_PV_ID","Role"));
		}
		
		ERole erole= new ERole();
		erole.setName(role.getName());
		erole.setDescription(role.getDescription());
		erole.setRoleId(role.getId());
		erole.setCategory(role.getCategory());
		
		if(findRoleById(role.getId()) == null){
			throw  new IOException(Messages.getString("GALAXY_MISSING","Role",role.getId().toString()));
		}
		
		manager.merge(erole);
		
	}

	
}
